//EShaderBone
#include "EShaderBone.h"
#include "../xmlnode/XmlNode.h"
#include "EDefine.h"
#include "XmlMesh.h"
#include "../GameTime.h"
#include "Shader.h"
#include "util.h"

EShaderBone::EShaderBone( LPDIRECT3DDEVICE9 pDevice ) : EMesh( pDevice )
{
	m_pIndex = NULL;
	m_pMeshVerts = NULL; 
	m_pBoneVerts = NULL;
}
//-----------------------------------------------------------------------------
EShaderBone::~EShaderBone()
{
	if( m_pMeshVerts )
	{
		delete [] m_pMeshVerts;
		m_pMeshVerts = NULL;
	}

	if( m_pBoneVerts )
	{
		delete [] m_pBoneVerts;
		m_pBoneVerts = NULL;
	}

	if( m_pIndex )
	{
		delete [] m_pIndex;
		m_pIndex = NULL;
	}
}
//-----------------------------------------------------------------------------
const char* EShaderBone::GetClassName()
{
	return "EShaderBone";
}
//-----------------------------------------------------------------------------
int	EShaderBone::CreateVertexBuffer( XmlNode* pXmlNode )
{
	XmlMesh* pXmlMesh = NULL;
	if( strcmp( pXmlNode->GetClassName(), "XmlMesh" ) == 0 )
		pXmlMesh = (XmlMesh*)pXmlNode;

	std::vector<VertexInfo>& xmlVx = pXmlMesh->m_vertexArr;

	int n = m_nVertex = (int)xmlVx.size();
	
	m_pBoneVerts = new ShaderBoneVertex[ m_nVertex ];
	for( int i = 0; i < n; ++i )
	{
		m_pBoneVerts[i].p.x = xmlVx[i].p.x;
		m_pBoneVerts[i].p.y = xmlVx[i].p.y;
		m_pBoneVerts[i].p.z = xmlVx[i].p.z;
	}


	//ε
	m_nTriangle = (DWORD)pXmlMesh->m_indexArr.size();
	m_pIndex = new ShaderFaceIncex [m_nTriangle];
	for( int i = 0 ; i <  (int)m_nTriangle; i++ )
	{
		m_pIndex[i].n[0] = pXmlMesh->m_indexArr[i].n[0];
		m_pIndex[i].n[1] = pXmlMesh->m_indexArr[i].n[1];
		m_pIndex[i].n[2] = pXmlMesh->m_indexArr[i].n[2];
	}

	return 1;
}
//-----------------------------------------------------------------------------
void EShaderBone::OnFrameMove()
{
	if( this->m_nAniSize > 0 )
	{
		int nFrame = GameTime::GetFrame();	
		float fWt = GameTime::GetWeightTime();
		D3DXMatrixIdentity( &m_matWorld );
		MatrixLerp(&m_matWorld, &m_aniArray[nFrame], &m_aniArray[nFrame + 1 ], fWt );
	}
	else
		D3DXMatrixIdentity(&m_matWorld);

	if( m_id >= 0 )
		m_matBlend[m_id] = m_matWorld;
}
//-----------------------------------------------------------------------------
int	EShaderBone::OnRender()
{
	return 0;
	if( m_pBoneVerts == NULL )
		return 0;

	m_pDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME);
	m_pDevice->SetSoftwareVertexProcessing(TRUE);
	m_pDevice->SetTexture(0, NULL);

	D3DXMATRIX	matView, matProj;
	LPD3DXEFFECT pShaderEffect = Shader::Instance()->GetEffect();
	HRESULT hr = pShaderEffect->SetTechnique("Tech");
	hr = m_pDevice->GetTransform(D3DTS_VIEW, &matView);
	hr = m_pDevice->GetTransform(D3DTS_PROJECTION, &matProj);

	pShaderEffect->SetMatrix("m_mtViw", &matView);	
	pShaderEffect->SetMatrix("m_mtPrj", &matProj);

	// 
	D3DXMATRIXA16 scaleMat;  D3DXMatrixIdentity( &scaleMat );  D3DXMatrixScaling( &scaleMat, 0.4f, 0.4f, 0.4f );
	pShaderEffect->SetMatrix("m_mtWld", &( m_matBlend[m_id] * scaleMat) ); //   

    UINT cPass;
    hr = pShaderEffect->Begin( &cPass, 0 );
	hr = pShaderEffect->BeginPass( 0 );

	hr = m_pDevice->SetVertexDeclaration( Shader::Instance()->GetBoneDecl() );

	m_pDevice->DrawIndexedPrimitiveUP( D3DPT_TRIANGLELIST, 
		 0, m_nVertex, m_nTriangle, m_pIndex, D3DFMT_INDEX16, m_pBoneVerts, sizeof(ShaderBoneVertex) );

	pShaderEffect->EndPass();
	pShaderEffect->End();

	m_pDevice->SetVertexDeclaration(NULL);
	m_pDevice->SetVertexShader(NULL);
	m_pDevice->SetPixelShader(NULL);
	m_pDevice->SetSoftwareVertexProcessing(FALSE);
	return 1;
}


