//ESoftwareBoneMesh.cpp
#include "ESoftwareBoneMesh.h"
#include "../xmlnode/XmlNode.h"
#include "EDefine.h"
#include "XmlMesh.h"
#include "../GameTime.h"

ESoftwareBoneMesh::ESoftwareBoneMesh( LPDIRECT3DDEVICE9 pDevice ) : EMesh( pDevice )
{
	m_pVerts = NULL;
	m_pIndex = NULL;
}
//-----------------------------------------------------------------------------
ESoftwareBoneMesh::~ESoftwareBoneMesh()
{
	if( m_pVerts )
	{
		delete [] m_pVerts;
		m_pVerts = NULL;
	}

	if( m_pIndex )
	{
		delete [] m_pIndex;
		m_pIndex = NULL;
	}
}
//-----------------------------------------------------------------------------
const char* ESoftwareBoneMesh::GetClassName()
{
	return "ESoftwareBoneMesh";
}
//-----------------------------------------------------------------------------
int ESoftwareBoneMesh::CreateVertexBuffer( XmlNode* pXmlNode )
{
	XmlMesh* pXmlMesh = NULL;
	if( strcmp( pXmlNode->GetClassName(), "XmlMesh" ) == 0 )
		pXmlMesh = (XmlMesh*)pXmlNode;

	std::vector<VertexInfo>& xmlVx = pXmlMesh->m_vertexArr;

	int n = m_nVertex = (int)xmlVx.size();
	for( int i = 0; i < n; ++i )
	{
		VertexInfo v;
		v.index = xmlVx[i].index;
		v.p = xmlVx[i].p;
		
		int count = (int)xmlVx[i].boneIndex.size();
		for( int j = 0; j < count; ++j )
		{
			v.boneIndex.push_back( xmlVx[i].boneIndex[j] );
			v.boneWeight.push_back( xmlVx[i].boneWeight[j] );
		}
		m_vertexArr.push_back( v );
	}

	m_pVerts = new SoftwareBoneVertex[ m_nVertex ];
	//ε
	m_nTriangle = (DWORD)pXmlMesh->m_indexArr.size();
	m_pIndex = new SoftwareFaceIncex [m_nTriangle];
	for( int i = 0 ; i <  (int)m_nTriangle; i++ )
	{
		m_pIndex[i].n[0] = pXmlMesh->m_indexArr[i].n[0];
		m_pIndex[i].n[1] = pXmlMesh->m_indexArr[i].n[1];
		m_pIndex[i].n[2] = pXmlMesh->m_indexArr[i].n[2];
	}

	return 1;
}
//-----------------------------------------------------------------------------
void ESoftwareBoneMesh::OnFrameMove()
{
	//Ʈ Ű ִϸ̼ 
	if( this->m_nAniSize > 0 )
	{
		int nFrame = GameTime::GetFrame();	
		m_matWorld = m_aniArray[nFrame];	
	}
	else
		D3DXMatrixIdentity(&m_matWorld);
}
//-----------------------------------------------------------------------------
void ESoftwareBoneMesh::ApplyVertexWeight()
{
	if( (int)m_vertexArr.size()  < 3 )
		return;

	int nCount = (int)m_vertexArr.size();
	for( int i = 0; i < nCount; ++i )
	{
		VertexInfo& vertexInfo = m_vertexArr[i];
		int boneCount = (int)vertexInfo.boneIndex.size();

		D3DXMATRIXA16	mat;
		memset(&mat, 0, sizeof(D3DXMATRIXA16));

		for( int j = 0; j < boneCount; ++j )
		{
			float fw = vertexInfo.boneWeight[j];
			int   id = vertexInfo.boneIndex[j];
			IndexENodeMap::iterator iter = m_indexMap.find( id );
			if( iter == m_indexMap.end() )
			{
				assert( 0 && "ε  ̴" );
			}
			ENode* eNode = iter->second;
			const D3DXMATRIXA16& worldMat = eNode->GetWorldMatrix();

			mat += ( worldMat * fw );
		}
		D3DXVECTOR3 v;
		v.x = vertexInfo.p.x;
		v.y = vertexInfo.p.y;
		v.z = vertexInfo.p.z;

		D3DXVec3TransformCoord(&m_pVerts[i].p, &v, &mat);
	}
}
//-----------------------------------------------------------------------------
int ESoftwareBoneMesh::OnRender()
{
#if  1	
	// 
	D3DXMATRIXA16 scaleMat;
	D3DXMatrixIdentity( &scaleMat );
	D3DXMatrixScaling( &scaleMat, 0.4f, 0.4f, 0.4f );

	ApplyVertexWeight();
	m_pDevice->SetTransform( D3DTS_WORLD, &(m_matWorld * scaleMat) );
#else
	ApplyVertexWeight();
	m_pDevice->SetTransform( D3DTS_WORLD, &m_matWorld );
#endif
	m_pDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME);
	m_pDevice->SetFVF( SoftwareBoneVertex::FVF );
	m_pDevice->DrawIndexedPrimitiveUP( D3DPT_TRIANGLELIST, 
		 0, m_nVertex, m_nTriangle, m_pIndex, D3DFMT_INDEX16, m_pVerts, sizeof(SoftwareBoneVertex) );
	return 1;
}



