//Shader.cpp
#include "Shader.h"
#include "EShaderBoneMesh.h"

Shader* Shader::m_pInstance = NULL;

Shader::Shader( LPDIRECT3DDEVICE9 pDevice )
{
	m_pMeshDecl = NULL;
	m_pBoneDecl = NULL;
	m_pShaderEffect = NULL;
	m_pDevice = pDevice;
}
//-----------------------------------------------------------------
Shader::~Shader()
{
	if( m_pMeshDecl )
		m_pMeshDecl->Release();
	m_pMeshDecl = NULL;

	if( m_pBoneDecl )
		m_pBoneDecl->Release();
	m_pBoneDecl = NULL;

	if( m_pShaderEffect )
		m_pShaderEffect->Release();
	m_pShaderEffect = NULL;
	m_pDevice = NULL;
}
//-----------------------------------------------------------------
bool Shader::CreateInstance( LPDIRECT3DDEVICE9 pDevice )
{
	Shader* pShader = new Shader( pDevice );

	bool bRes = pShader->Init();
	if( bRes == false )
		delete pShader;
	else
		m_pInstance = pShader;

	return  bRes;
}
//-----------------------------------------------------------------
bool Shader::CreateDeclaration()
{
	// 
	D3DVERTEXELEMENT9 vertexDecl[MAX_FVF_DECL_SIZE] = {0};
	D3DXDeclaratorFromFVF( ShaderBlendVertex::FVF, vertexDecl );

	LPDIRECT3DVERTEXDECLARATION9	pFVF = NULL;
	
	if( FAILED(m_pDevice->CreateVertexDeclaration(vertexDecl, &pFVF)) )
		return false;

	if( m_pMeshDecl )
		m_pMeshDecl->Release();
	m_pMeshDecl = pFVF;

	//  
	D3DVERTEXELEMENT9 boneVertexDecl[MAX_FVF_DECL_SIZE] = {0};
	D3DXDeclaratorFromFVF( ShaderBoneVertex::FVF, boneVertexDecl );

	pFVF = NULL;		
			
	if( FAILED(m_pDevice->CreateVertexDeclaration(boneVertexDecl, &pFVF)) )
		return false;

	if( m_pBoneDecl )
		m_pBoneDecl->Release();
	m_pBoneDecl = pFVF;

	return true;
}
//-----------------------------------------------------------------
void Shader::ReleaseInstance()
{
	if( m_pInstance )
		delete m_pInstance;
	m_pInstance = NULL;
}
//-----------------------------------------------------------------
bool Shader::Init()
{
	DWORD dwFlags = 0;
#if defined( _DEBUG )
	dwFlags |= D3DXSHADER_DEBUG;
#endif
	LPD3DXBUFFER ppCompilationErrors = NULL;

	HRESULT hr = D3DXCreateEffectFromFile( m_pDevice
		, "data/shader.fx"
		, NULL
		, NULL
		, dwFlags
		, NULL
		, &m_pShaderEffect
		, &ppCompilationErrors);
	
	if( FAILED(hr) )
	{
		MessageBox( NULL, (char*)ppCompilationErrors->GetBufferPointer(), "Err", 0);
		ppCompilationErrors->Release();
		return false;
	}

	CreateDeclaration();
	return true;

}
//-----------------------------------------------------------------
void Shader::LostDevice()
{
	m_pShaderEffect->OnLostDevice();
}
//-----------------------------------------------------------------
void Shader::Restore()
{
	m_pShaderEffect->OnResetDevice();	
}