//XmlMesh.cpp
#include "XmlMesh.h"
#include "../xmlnode/XmlNode.h"
#include "../xmlnode/StringUtil.h"
#include "EDefine.h"
#include "EStaticMesh.h"
#include "EHardwareBoneMesh.h"
#include "EHardwareBone.h"
#include "EShaderBoneMesh.h"
#include "EShaderBone.h"


extern LPDIRECT3DDEVICE9   g_d3d_Device; 

XmlMesh::XmlMesh()
{
	m_bUseData = true;
	D3DXMatrixIdentity( &m_matLocal );
	m_pAniInfo = NULL;
	m_aniArray = NULL;
	m_nAniSize = 0;
	m_id = -1;
}
//-----------------------------------------------------------------------------
int GetIntAttribute( XmlMesh* pXml, const char* szName )
{
	std::string* pStr = pXml->GetAttribute( szName );
	if( pStr == NULL )
		return -1;
	int number = ToInt( *pStr );
	return number;
}
//-----------------------------------------------------------------------------
XmlMesh::~XmlMesh()
{
	m_vertexArr.clear();
	m_indexArr.clear();
	if( m_pAniInfo )
	{
		delete m_pAniInfo;
		m_pAniInfo = NULL;
	}
	if( m_aniArray )
	{
		delete [] m_aniArray;
		m_aniArray = NULL;
		m_nAniSize = 0;
	}

}
//-----------------------------------------------------------------------------
const char*	XmlMesh::GetClassName()
{
	return "XmlMesh";
}
//-----------------------------------------------------------------------------
void XmlMesh::SetUseData( bool bFlag )
{
	m_bUseData = bFlag;
}
//-----------------------------------------------------------------------------
bool XmlMesh::GetUseData()
{
	return m_bUseData;
}
//-----------------------------------------------------------------------------
void XmlMesh::SetUseDataChild( bool bFlag )
{
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();

	for( ; iter != endIter; ++iter )
	{
		XmlMesh* pChild = (XmlMesh*)*iter;
		pChild->SetUseData( bFlag );
	}
}
//-----------------------------------------------------------------------------
XmlMesh* XmlMesh::GetParent()
{
	return (XmlMesh*)m_pParent;
}
//-----------------------------------------------------------------------------
void XmlMesh::ConvertMeshInfo()
{
	SetUseDataChild( false );

	//  б
	XmlNode* pVertexXmlNode = FindFirstChild( "Vertex" );
	if( pVertexXmlNode == NULL )
		return;
	XmlMesh* pXmlMesh = (XmlMesh*)pVertexXmlNode;
	pXmlMesh->FetchVertex( GetParent()->m_vertexArr );
	
	// ε Ʈ б
	XmlNode* pFaceXmlNode = FindFirstChild( "Face" );
	if( pFaceXmlNode  == NULL )
		return;
	pXmlMesh = (XmlMesh*)pFaceXmlNode;
	pXmlMesh->FetchFace( GetParent()->m_indexArr );

	//  ε б
	XmlNode* pBoneIndexXmlNode = FindFirstChild( "boneIndex" );
	if( pBoneIndexXmlNode  == NULL )
		return;
	pXmlMesh = (XmlMesh*)pBoneIndexXmlNode;
	pXmlMesh->FetchBoneIndex( GetParent()->m_vertexArr );

	//  Weight б
	XmlNode* pWeightXmlNode = FindFirstChild( "weight" );
	if( pWeightXmlNode  == NULL )
		return;
	pXmlMesh = (XmlMesh*)pWeightXmlNode;
	pXmlMesh->FetchBoneWeight( GetParent()->m_vertexArr );
}
//-----------------------------------------------------------------------------
void XmlMesh::ConvertTransformInfo()
{
	SetUseData( false );
	XmlNode* pLocalXmlNode = FindFirstChild( "LocalMatrix" );
	if( pLocalXmlNode == NULL )
		return;

	XmlMesh* pXmlMesh = (XmlMesh*)pLocalXmlNode;
	base::Matrix mat = ToMatrix( *pXmlMesh->GetData() );

	//"Transform" XML  Ŀ ȴ.
	if( strcmp( m_pParent->GetClassName(), "XmlMesh" ) == 0 )
	{
		XmlMesh* pParent = (XmlMesh*)m_pParent;
		ToDXMatrix( pParent->m_matLocal, mat ); 
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::ConvertAnimationHeader()
{
	SetUseData( false );
	XmlNode* pXmlNode = FindFirstChild( "beginFrame" );
	if( pXmlNode == NULL )
		return;

	XmlMesh* pRoot = (XmlMesh*)GetRoot();
	assert( pRoot->m_pAniInfo == NULL );
	pRoot->m_pAniInfo = new AnimationInfo;
	AnimationInfo* pAniInfo = pRoot->m_pAniInfo;

	pAniInfo->beginFrame = ToInt( *pXmlNode->GetData() );
	pAniInfo->endFrame = ToInt( *FindFirstChild( "endFrame" )->GetData() );
	pAniInfo->fps = ToInt( *FindFirstChild( "fps" )->GetData() );
	pAniInfo->tickPerFrame = ToInt( *FindFirstChild( "tickPerFrame" )->GetData() );
}
//-----------------------------------------------------------------------------
void XmlMesh::ConvertAnimation()
{
	SetUseData( false );
	int num = 0;
	std::string* pStr = GetAttribute( "count" );
	if( pStr )
		num = ToInt( *pStr );

	if( num == 0 )
		return;

	XmlNodeVector nodeArr;
	FindChild( "matrix", nodeArr );
	if( nodeArr.size() != num )
		return;

	XmlMesh* pXmlMesh = (XmlMesh*)m_pParent;
	pXmlMesh->m_nAniSize = num;
	pXmlMesh->m_aniArray = new D3DXMATRIXA16[num];

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for( int index = 0; iter != endIter; ++iter, ++index )
	{
		XmlNode* xmlNode = *iter;
		base::Matrix mat = ToMatrix( *xmlNode->GetData() );		
		ToDXMatrix( pXmlMesh->m_aniArray[index], mat );
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::ToDXMatrix( D3DXMATRIXA16& destMat, const base::Matrix& srcMat )
{
	destMat._11 = srcMat._11;	destMat._12 = srcMat._12;	destMat._13 = srcMat._13;	destMat._14 = srcMat._14;
	destMat._21 = srcMat._21;	destMat._22 = srcMat._22;	destMat._23 = srcMat._23;	destMat._24 = srcMat._24;
	destMat._31 = srcMat._31;	destMat._32 = srcMat._32;	destMat._33 = srcMat._33;	destMat._34 = srcMat._34;
	destMat._41 = srcMat._41;	destMat._42 = srcMat._42;	destMat._43 = srcMat._43;	destMat._44 = srcMat._44;
}
//-----------------------------------------------------------------------------
void XmlMesh::OnUpdate()
{
	m_id = GetIntAttribute( this, "index" );

	if( *GetName() == "mesh" )
		ConvertMeshInfo(); 	
	else if( *GetName() == "Transform" )
		ConvertTransformInfo(); 
	else if( *GetName() == "AnimationHeader" )
		ConvertAnimationHeader();
	else if( *GetName() == "Animation" )
		ConvertAnimation();
}
//-----------------------------------------------------------------------------
void XmlMesh::FetchVertex( std::vector<VertexInfo>& vertexArr )
{
	//"Vertex" Ʈ - "count" Ӽ ϱ
	int number = GetIntAttribute( this, "count" );
	if( number < 3 )
		return;

	//"Vertex" Ʈ "data" Ʈ ϱ
	XmlNodeVector nodeArr;
	FindChild( "data", nodeArr );
	if( nodeArr.size() != number )
		return;

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for( int index = 0; iter != endIter; ++iter, ++index )
	{
		XmlNode* xmlNode = *iter;

		VertexInfo info;
		info.index = index;
		info.p = ToVector3( *xmlNode->GetData() );		
		vertexArr.push_back( info );
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::FetchFace( std::vector<base::Index3>& indexArr )
{
	//"Face" Ʈ - "Number" Ʈ ϱ
	int number = GetIntAttribute( this, "count" );
	if( number < 1 )
		return;

	//"Face" Ʈ "data" Ʈ ϱ
	XmlNodeVector nodeArr;
	FindChild( "data", nodeArr );
	if( nodeArr.size() != number )
		return;

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for(; iter != endIter; ++iter )
	{
		XmlNode* xmlNode = *iter;
		base::Index3 index = ToIndex3( *xmlNode->GetData() );
		indexArr.push_back( index );
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::FetchBoneIndex( std::vector<VertexInfo>& vertexArr )
{
	//"boneIndex" Ʈ - "count" Ӽ ϱ
	int number = GetIntAttribute( this, "count" );
	if( (int)vertexArr.size() != number )
		return;

	//"Vertex" Ʈ "data" Ʈ ϱ
	XmlNodeVector nodeArr;
	FindChild( "data", nodeArr );
	if( nodeArr.size() != number )
		return;

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for( int index = 0; iter != endIter; ++iter, ++index )
	{
		XmlNode* xmlNode = *iter;
		ToIntArray( vertexArr[index].boneIndex, *xmlNode->GetData() );
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::FetchBoneWeight( std::vector<VertexInfo>& vertexArr )
{
	//"weight" Ʈ - "count" Ӽ ϱ
	int number = GetIntAttribute( this, "count" );
	if( (int)vertexArr.size() != number )
		return;

	//"Vertex" Ʈ "data" Ʈ ϱ
	XmlNodeVector nodeArr;
	FindChild( "data", nodeArr );
	if( nodeArr.size() != number )
		return;

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for( int index = 0; iter != endIter; ++iter, ++index )
	{
		XmlNode* xmlNode = *iter;
		ToFloatArray( vertexArr[index].boneWeight, *xmlNode->GetData() );
	}
}
//-----------------------------------------------------------------------------
ENode* XmlMesh::OnXmlToENode( ENode* pParent )
{
	ENode* pNode = NULL;

	std::string* boneStr =  this->GetAttribute( "bone" );
	bool bBone = false;
	if( boneStr )
		bBone = ToBool( *boneStr );

	if( m_vertexArr.size() > 0 )
	{
		//EStaticMesh* pMesh = new EStaticMesh( g_d3d_Device );
		if( bBone )
		{
			EShaderBone* pMesh = new EShaderBone( g_d3d_Device );
			pMesh->CreateVertexBuffer( this );
			pNode = pMesh;		
		}
		else
		{
			EShaderBoneMesh* pMesh = new EShaderBoneMesh( g_d3d_Device );
			pMesh->CreateVertexBuffer( this );
			pNode = pMesh;
		}
	}
	else 
	{
		pNode = new ENode( g_d3d_Device );
	}

	pNode->SetBone( bBone );
	if( pParent == NULL )
		pNode->CreateAnimationHeader( this );

	pNode->SetClassType( this->GetName()->c_str() );
	pNode->SetName( this->GetAttribute("name")->c_str() );
	pNode->SetMatrix( m_matLocal );
	pNode->SetID( m_id );

	ENode::m_indexMap.insert( std::make_pair( m_id, pNode ) );

	if( m_aniArray )
		pNode->CreateAnimation( this );
	return pNode;
}
//-----------------------------------------------------------------------------
void XmlMesh::CreateMesh( ENode*& pParent )
{
	if( GetUseData() == false )
		return;

	ENode* pNode;
	pNode = OnXmlToENode( pParent );
	if( pParent == NULL )
	{
		pParent = pNode;
	}
	else
	{
		pParent->AttachChild( pNode );
	}

	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();

	for( ; iter != endIter; ++iter )
	{
		XmlMesh* pChild = (XmlMesh*)*iter;
		pChild->CreateMesh( pNode );
	}	
}
